@@ -16,18 +16,12 @@
  */
 package org.apache.log4j.builders.filter;
 
-import org.apache.log4j.config.PropertiesConfiguration;
+import org.apache.log4j.builders.Parser;
 import org.apache.log4j.spi.Filter;
-import org.apache.log4j.xml.XmlConfiguration;
-import org.w3c.dom.Element;
 
 /**
  * Define a Filter Builder.
  */
-public interface FilterBuilder {
-
-    Filter parseFilter(Element element, XmlConfiguration config);
-
-    Filter parseFilter(PropertiesConfiguration config);
-
+public interface FilterBuilder extends Parser<Filter> {
+    // empty
 }
