@@ -16,21 +16,21 @@
  */
 package org.apache.logging.log4j.util;
 
+import java.util.ArrayDeque;
+import java.util.Collections;
+import java.util.Deque;
 import java.util.List;
-import java.util.Stack;
-import java.util.function.Function;
 import java.util.function.Predicate;
 import java.util.stream.Collectors;
-import java.util.stream.Stream;
 
 /**
  * <em>Consider this class private.</em> Determines the caller's class.
  */
 public class StackLocator {
 
-    private final static StackWalker walker = StackWalker.getInstance(StackWalker.Option.RETAIN_CLASS_REFERENCE);
+    private final static StackWalker WALKER = StackWalker.getInstance(StackWalker.Option.RETAIN_CLASS_REFERENCE);
 
-    private final static StackWalker stackWalker = StackWalker.getInstance();
+    private final static StackWalker STACK_WALKER = StackWalker.getInstance();
 
     private final static StackLocator INSTANCE = new StackLocator();
 
@@ -48,7 +48,7 @@ public class StackLocator {
         if (callerPredicate == null) {
             throw new IllegalArgumentException("callerPredicate cannot be null");
         }
-        return walker.walk(s -> s
+        return WALKER.walk(s -> s
                         .map(StackWalker.StackFrame::getDeclaringClass)
                         // Skip until the sentinel class is found
                         .dropWhile(clazz -> !sentinelClass.equals(clazz))
@@ -62,7 +62,7 @@ public class StackLocator {
     }
 
     public Class<?> getCallerClass(final String fqcn, final String pkg) {
-        return walker.walk(s -> s
+        return WALKER.walk(s -> s
                 .dropWhile(f -> !f.getClassName().equals(fqcn))
                 .dropWhile(f -> f.getClassName().equals(fqcn))
                 .dropWhile(f -> !f.getClassName().startsWith(pkg))
@@ -72,35 +72,36 @@ public class StackLocator {
     }
 
     public Class<?> getCallerClass(final Class<?> anchor) {
-        return walker.walk(s -> s.dropWhile(f -> !f.getDeclaringClass().equals(anchor)).
-                dropWhile(f -> f.getDeclaringClass().equals(anchor)).findFirst()).
-                map(StackWalker.StackFrame::getDeclaringClass).orElse(null);
+        return WALKER.walk(s -> s.dropWhile(f -> !f.getDeclaringClass().equals(anchor))
+                     .dropWhile(f -> f.getDeclaringClass().equals(anchor)).findFirst())
+                     .map(StackWalker.StackFrame::getDeclaringClass).orElse(null);
     }
 
     public Class<?> getCallerClass(final int depth) {
-        return walker.walk(s -> s.skip(depth).findFirst()).map(StackWalker.StackFrame::getDeclaringClass).orElse(null);
+        return WALKER.walk(s -> s.skip(depth).findFirst()).map(StackWalker.StackFrame::getDeclaringClass).orElse(null);
     }
 
-    public Stack<Class<?>> getCurrentStackTrace() {
+    public Deque<Class<?>> getCurrentStackTrace() {
         // benchmarks show that using the SecurityManager is much faster than looping through getCallerClass(int)
         if (PrivateSecurityManagerStackTraceUtil.isEnabled()) {
             return PrivateSecurityManagerStackTraceUtil.getCurrentStackTrace();
         }
-        Stack<Class<?>> stack = new Stack<Class<?>>();
-        List<Class<?>> classes = walker.walk(s -> s.map(f -> f.getDeclaringClass()).collect(Collectors.toList()));
-        stack.addAll(classes);
-        return stack;
+        final Deque<Class<?>> stack = new ArrayDeque<Class<?>>();
+        return WALKER.walk(s -> {
+            s.forEach(f -> stack.add(f.getDeclaringClass()));
+            return stack;
+        });
     }
 
     public StackTraceElement calcLocation(final String fqcnOfLogger) {
-        return stackWalker.walk(
+        return STACK_WALKER.walk(
                 s -> s.dropWhile(f -> !f.getClassName().equals(fqcnOfLogger)) // drop the top frames until we reach the logger
-                        .dropWhile(f -> f.getClassName().equals(fqcnOfLogger)) // drop the logger frames
-                        .findFirst()).map(StackWalker.StackFrame::toStackTraceElement).orElse(null);
+                      .dropWhile(f -> f.getClassName().equals(fqcnOfLogger)) // drop the logger frames
+                      .findFirst()).map(StackWalker.StackFrame::toStackTraceElement).orElse(null);
     }
 
     public StackTraceElement getStackTraceElement(final int depth) {
-        return stackWalker.walk(s -> s.skip(depth).findFirst())
+        return STACK_WALKER.walk(s -> s.skip(depth).findFirst())
                 .map(StackWalker.StackFrame::toStackTraceElement).orElse(null);
     }
 }
