@@ -59,6 +59,16 @@ public class SortedArrayStringMapTest {
         assertEquals(0, new SortedArrayStringMap((SortedArrayStringMap) null).size());
     }
 
+    @Test
+    public void testConstructorNonStringKeys() {
+        Map<Object, Integer> map = new HashMap<>(1);
+        map.put(Long.MAX_VALUE, 1);
+        map.put(null, null);
+        SortedArrayStringMap sMap = new SortedArrayStringMap((Map) map);
+        assertEquals(1, (int) sMap.getValue(Long.toString(Long.MAX_VALUE)));
+        assertEquals((Integer) null, sMap.getValue(null));
+    }
+
     @Test
     public void testToString() {
         final SortedArrayStringMap original = new SortedArrayStringMap();
@@ -72,7 +82,7 @@ public class SortedArrayStringMapTest {
     public void testSerialization() throws Exception {
         final SortedArrayStringMap original = new SortedArrayStringMap();
         original.putValue("a", "avalue");
-        original.putValue("B", "Bvalue");
+        original.putValue("B", null); // null may be treated differently
         original.putValue("3", "3value");
 
         final byte[] binary = serialize(original);
@@ -80,6 +90,14 @@ public class SortedArrayStringMapTest {
         assertEquals(original, copy);
     }
 
+    @Test
+    public void testSerializationOfEmptyMap() throws Exception {
+        final SortedArrayStringMap original = new SortedArrayStringMap();
+        final byte[] binary = serialize(original);
+        final SortedArrayStringMap copy = deserialize(binary);
+        assertEquals(original, copy);
+    }
+
     @Test
     public void testSerializationOfNonSerializableValue() throws Exception {
         final SortedArrayStringMap original = new SortedArrayStringMap();
@@ -170,8 +188,7 @@ public class SortedArrayStringMapTest {
     private SortedArrayStringMap deserialize(final byte[] binary) throws IOException, ClassNotFoundException {
         final ByteArrayInputStream inArr = new ByteArrayInputStream(binary);
         try (final ObjectInputStream in = new FilteredObjectInputStream(inArr)) {
-            final SortedArrayStringMap result = (SortedArrayStringMap) in.readObject();
-            return result;
+            return (SortedArrayStringMap) in.readObject();
         }
     }
 
@@ -393,12 +410,17 @@ public class SortedArrayStringMapTest {
     @Test
     public void testEquals() {
         final SortedArrayStringMap original = new SortedArrayStringMap();
+        final SortedArrayStringMap other = new SortedArrayStringMap();
+
+        assertEquals(other, original, "Empty maps are equal");
+        assertEquals(other.hashCode(), original.hashCode(), "Empty maps have equal hashcode");
+        assertNotEquals(original, "Object other than SortedArrayStringMap");
+
         original.putValue("a", "avalue");
         original.putValue("B", "Bvalue");
         original.putValue("3", "3value");
         assertEquals(original, original); // equal to itself
 
-        final SortedArrayStringMap other = new SortedArrayStringMap();
         other.putValue("a", "avalue");
         assertNotEquals(original, other);
 
@@ -407,15 +429,23 @@ public class SortedArrayStringMapTest {
 
         other.putValue("3", "3value");
         assertEquals(original, other);
+        assertEquals(original.hashCode(), other.hashCode());
 
         other.putValue("3", "otherValue");
         assertNotEquals(original, other);
 
         other.putValue("3", null);
         assertNotEquals(original, other);
+        assertNotEquals(original.hashCode(), other.hashCode());
 
         other.putValue("3", "3value");
         assertEquals(original, other);
+        assertEquals(other, original); // symmetry
+
+        original.putValue("key not in other", "4value");
+        other.putValue("key not in original", "4value");
+        assertNotEquals(original, other);
+        assertNotEquals(other, original); // symmetry
     }
 
     @Test
