@@ -41,11 +41,9 @@ import org.apache.logging.log4j.core.config.DefaultConfiguration;
 import org.apache.logging.log4j.core.config.NullConfiguration;
 import org.apache.logging.log4j.core.config.Reconfigurable;
 import org.apache.logging.log4j.core.impl.Log4jLogEvent;
-import org.apache.logging.log4j.core.impl.ThreadContextDataInjector;
 import org.apache.logging.log4j.core.jmx.Server;
 import org.apache.logging.log4j.core.util.Cancellable;
 import org.apache.logging.log4j.core.util.ExecutorServices;
-import org.apache.logging.log4j.core.util.Loader;
 import org.apache.logging.log4j.core.util.NetUtils;
 import org.apache.logging.log4j.core.util.ShutdownCallbackRegistry;
 import org.apache.logging.log4j.message.MessageFactory;
@@ -68,15 +66,6 @@ public class LoggerContext extends AbstractLifeCycle
         implements org.apache.logging.log4j.spi.LoggerContext, AutoCloseable, Terminable, ConfigurationListener,
         LoggerContextShutdownEnabled {
 
-    static {
-        try {
-            // LOG4J2-1642 preload ExecutorServices as it is used in shutdown hook
-            Loader.loadClass(ExecutorServices.class.getName());
-        } catch (final Exception e) {
-            LOGGER.error("Failed to preload ExecutorServices class.", e);
-        }
-    }
-
     /**
      * Property name of the property change event fired if the configuration is changed.
      */
@@ -135,9 +124,6 @@ public class LoggerContext extends AbstractLifeCycle
             externalMap.put(EXTERNAL_CONTEXT_KEY, externalContext);
         }
         this.configLocation = configLocn;
-        Thread runner = new Thread(new ThreadContextDataTask(), "Thread Context Data Task");
-        runner.setDaemon(true);
-        runner.start();
     }
 
     /**
@@ -166,9 +152,6 @@ public class LoggerContext extends AbstractLifeCycle
         } else {
             configLocation = null;
         }
-        Thread runner = new Thread(new ThreadContextDataTask(), "Thread Context Data Task");
-        runner.setDaemon(true);
-        runner.start();
     }
 
     @Override
@@ -308,6 +291,8 @@ public class LoggerContext extends AbstractLifeCycle
             final LoggerContextFactory factory = LogManager.getFactory();
             if (factory instanceof ShutdownCallbackRegistry) {
                 LOGGER.debug(SHUTDOWN_HOOK_MARKER, "Shutdown hook enabled. Registering a new one.");
+                // LOG4J2-1642 preload ExecutorServices as it is used in shutdown hook
+                ExecutorServices.ensureInitialized();
                 try {
                     final long shutdownTimeoutMillis = this.configuration.getShutdownTimeoutMillis();
                     this.shutdownCallback = ((ShutdownCallbackRegistry) factory).addShutdownCallback(new Runnable() {
@@ -534,6 +519,16 @@ public class LoggerContext extends AbstractLifeCycle
         return loggerRegistry.getLogger(name, messageFactory);
     }
 
+    /**
+     * Gets the LoggerRegistry.
+     *
+     * @return the LoggerRegistry.
+     * @since 2.17.2
+     */
+    public LoggerRegistry<Logger> getLoggerRegistry() {
+        return loggerRegistry;
+    }
+
     /**
      * Determines if the specified Logger exists.
      *
@@ -776,14 +771,4 @@ public class LoggerContext extends AbstractLifeCycle
     protected Logger newInstance(final LoggerContext ctx, final String name, final MessageFactory messageFactory) {
         return new Logger(ctx, name, messageFactory);
     }
-
-    private class ThreadContextDataTask implements Runnable {
-
-        @Override
-        public void run() {
-            LOGGER.debug("Initializing Thread Context Data Service Providers");
-            ThreadContextDataInjector.initServiceProviders();
-            LOGGER.debug("Thread Context Data Service Provider initialization complete");
-        }
-    }
 }
