@@ -36,6 +36,7 @@ import java.nio.file.attribute.PosixFilePermissions;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.Objects;
 import java.util.Set;
 
 import org.apache.logging.log4j.core.Layout;
@@ -188,23 +189,28 @@ public class FileManager extends OutputStreamManager {
         final String filename = getFileName();
         LOGGER.debug("Now writing to {} at {}", filename, new Date());
         final File file = new File(filename);
+        createParentDir(file);
         final FileOutputStream fos = new FileOutputStream(file, isAppend);
         if (file.exists() && file.length() == 0) {
             try {
                 FileTime now = FileTime.fromMillis(System.currentTimeMillis());
                 Files.setAttribute(file.toPath(), "creationTime", now);
             } catch (Exception ex) {
-                LOGGER.warn("Unable to set current file tiem for {}", filename);
+                LOGGER.warn("Unable to set current file time for {}", filename);
             }
+            writeHeader(fos);
         }
         defineAttributeView(Paths.get(filename));
         return fos;
     }
 
+    protected void createParentDir(File file) {
+    }
+
     protected void defineAttributeView(final Path path) {
         if (attributeViewEnabled) {
             try {
-                // FileOutputStream may not create new file on all jvm
+                // FileOutputStream may not create new file on all JVM
                 path.toFile().createNewFile();
 
                 FileUtils.defineFilePosixAttributeView(path, filePermissions, fileOwner, fileGroup);
@@ -314,9 +320,9 @@ public class FileManager extends OutputStreamManager {
     }
 
     /**
-     * Returns posix file permissions if defined and the OS supports posix file attribute,
+     * Returns POSIX file permissions if defined and the OS supports POSIX file attribute,
      * null otherwise.
-     * @return File posix permissions
+     * @return File POSIX permissions
      * @see PosixFileAttributeView
      */
     public Set<PosixFilePermission> getFilePermissions() {
@@ -334,7 +340,7 @@ public class FileManager extends OutputStreamManager {
     }
 
     /**
-     * Returns file group if defined and the OS supports posix/group file attribute view,
+     * Returns file group if defined and the OS supports POSIX/group file attribute view,
      * null otherwise.
      * @return File group
      * @see PosixFileAttributeView
@@ -346,7 +352,7 @@ public class FileManager extends OutputStreamManager {
     /**
      * Returns true if file attribute view enabled for this file manager.
      *
-     * @return True if posix or owner supported and defined false otherwise.
+     * @return True if POSIX or owner supported and defined false otherwise.
      */
     public boolean isAttributeViewEnabled() {
         return attributeViewEnabled;
@@ -424,6 +430,7 @@ public class FileManager extends OutputStreamManager {
          */
         @Override
         public FileManager createManager(final String name, final FactoryData data) {
+            Objects.requireNonNull(name, "filename is missing");
             final File file = new File(name);
             try {
                 FileUtils.makeParentDirs(file);
