@@ -130,46 +130,100 @@ public class SocketAppender extends AbstractOutputStreamAppender<AbstractSocketM
             return immediateFail;
         }
 
+        public B setAdvertise(final boolean advertise) {
+            this.advertise = advertise;
+            return asBuilder();
+        }
+
+        public B setConnectTimeoutMillis(final int connectTimeoutMillis) {
+            this.connectTimeoutMillis = connectTimeoutMillis;
+            return asBuilder();
+        }
+
+        public B setHost(final String host) {
+            this.host = host;
+            return asBuilder();
+        }
+
+        public B setImmediateFail(final boolean immediateFail) {
+            this.immediateFail = immediateFail;
+            return asBuilder();
+        }
+
+        public B setPort(final int port) {
+            this.port = port;
+            return asBuilder();
+        }
+
+        public B setProtocol(final Protocol protocol) {
+            this.protocol = protocol;
+            return asBuilder();
+        }
+
+        public B setReconnectDelayMillis(final int reconnectDelayMillis) {
+            this.reconnectDelayMillis = reconnectDelayMillis;
+            return asBuilder();
+        }
+
+        public B setSocketOptions(final SocketOptions socketOptions) {
+            this.socketOptions = socketOptions;
+            return asBuilder();
+        }
+
+        public B setSslConfiguration(final SslConfiguration sslConfiguration) {
+            this.sslConfiguration = sslConfiguration;
+            return asBuilder();
+        }
+
+        @Deprecated
         public B withAdvertise(final boolean advertise) {
             this.advertise = advertise;
             return asBuilder();
         }
 
+        @Deprecated
         public B withConnectTimeoutMillis(final int connectTimeoutMillis) {
             this.connectTimeoutMillis = connectTimeoutMillis;
             return asBuilder();
         }
 
+        @Deprecated
         public B withHost(final String host) {
             this.host = host;
             return asBuilder();
         }
 
+        @Deprecated
         public B withImmediateFail(final boolean immediateFail) {
             this.immediateFail = immediateFail;
             return asBuilder();
         }
 
+        @Deprecated
         public B withPort(final int port) {
             this.port = port;
             return asBuilder();
         }
 
+        @Deprecated
         public B withProtocol(final Protocol protocol) {
             this.protocol = protocol;
             return asBuilder();
         }
 
+        @Deprecated
         public B withReconnectDelayMillis(final int reconnectDelayMillis) {
             this.reconnectDelayMillis = reconnectDelayMillis;
             return asBuilder();
         }
 
+        @Deprecated
         public B withSocketOptions(final SocketOptions socketOptions) {
             this.socketOptions = socketOptions;
             return asBuilder();
         }
 
+        @Deprecated
         public B withSslConfiguration(final SslConfiguration sslConfiguration) {
             this.sslConfiguration = sslConfiguration;
             return asBuilder();
@@ -329,15 +383,15 @@ public class SocketAppender extends AbstractOutputStreamAppender<AbstractSocketM
 
         // @formatter:off
         return newBuilder()
-        .withAdvertise(advertise)
+        .setAdvertise(advertise)
         .setConfiguration(configuration)
-        .withConnectTimeoutMillis(connectTimeoutMillis).setFilter(filter)
-            .withHost(host).setIgnoreExceptions(ignoreExceptions)
-            .withImmediateFail(immediateFail).setLayout(layout).setName(name)
-            .withPort(port)
-            .withProtocol(protocol)
-            .withReconnectDelayMillis(reconnectDelayMillis)
-            .withSslConfiguration(sslConfig)
+        .setConnectTimeoutMillis(connectTimeoutMillis).setFilter(filter)
+            .setHost(host).setIgnoreExceptions(ignoreExceptions)
+            .setImmediateFail(immediateFail).setLayout(layout).setName(name)
+            .setPort(port)
+            .setProtocol(protocol)
+            .setReconnectDelayMillis(reconnectDelayMillis)
+            .setSslConfiguration(sslConfig)
             .build();
         // @formatter:on
     }
