@@ -29,6 +29,7 @@ import java.nio.file.attribute.FileTime;
 import java.util.Collection;
 import java.util.Date;
 import java.util.concurrent.ArrayBlockingQueue;
+import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.ThreadPoolExecutor;
@@ -67,10 +68,11 @@ public class RollingFileManager extends FileManager {
     private final Log4jThreadFactory threadFactory = Log4jThreadFactory.createThreadFactory("RollingFileManager");
     private volatile TriggeringPolicy triggeringPolicy;
     private volatile RolloverStrategy rolloverStrategy;
-    private volatile boolean renameEmptyFiles = false;
-    private volatile boolean initialized = false;
+    private volatile boolean renameEmptyFiles;
+    private volatile boolean initialized;
     private volatile String fileName;
-    private final FileExtension fileExtension;
+    private final boolean directWrite;
+    private final CopyOnWriteArrayList<RolloverListener> rolloverListeners = new CopyOnWriteArrayList<>();
 
     /* This executor pool will create a new Thread for every work async action to be performed. Using it allows
        us to make sure all the Threads are completed when the Manager is stopped. */
@@ -109,7 +111,7 @@ public class RollingFileManager extends FileManager {
         this.patternProcessor = new PatternProcessor(pattern);
         this.patternProcessor.setPrevFileTime(initialTime);
         this.fileName = fileName;
-        this.fileExtension = FileExtension.lookupForFile(pattern);
+        this.directWrite = rolloverStrategy instanceof DirectWriteRolloverStrategy;
     }
 
     @Deprecated
@@ -126,7 +128,7 @@ public class RollingFileManager extends FileManager {
         this.patternProcessor = new PatternProcessor(pattern);
         this.patternProcessor.setPrevFileTime(initialTime);
         this.fileName = fileName;
-        this.fileExtension = FileExtension.lookupForFile(pattern);
+        this.directWrite = rolloverStrategy instanceof DirectWriteRolloverStrategy;
     }
 
     /**
@@ -142,12 +144,12 @@ public class RollingFileManager extends FileManager {
 			advertiseURI, layout, filePermissions, fileOwner, fileGroup, writeHeader, buffer);
         this.size = size;
         this.initialTime = initialTime;
-        this.triggeringPolicy = triggeringPolicy;
-        this.rolloverStrategy = rolloverStrategy;
         this.patternProcessor = new PatternProcessor(pattern);
         this.patternProcessor.setPrevFileTime(initialTime);
+        this.triggeringPolicy = triggeringPolicy;
+        this.rolloverStrategy = rolloverStrategy;
         this.fileName = fileName;
-        this.fileExtension = FileExtension.lookupForFile(pattern);
+        this.directWrite = rolloverStrategy instanceof DirectFileRolloverStrategy;
     }
 
     public void initialize() {
@@ -155,11 +157,21 @@ public class RollingFileManager extends FileManager {
         if (!initialized) {
             LOGGER.debug("Initializing triggering policy {}", triggeringPolicy);
             initialized = true;
+            // LOG4J2-2981 - set the file size before initializing the triggering policy.
+            if (directWrite) {
+                // LOG4J2-2485: Initialize size from the most recently written file.
+                File file = new File(getFileName());
+                if (file.exists()) {
+                    size = file.length();
+                } else {
+                    ((DirectFileRolloverStrategy) rolloverStrategy).clearCurrentFileName();
+                }
+            }
             triggeringPolicy.initialize(this);
             if (triggeringPolicy instanceof LifeCycle) {
                 ((LifeCycle) triggeringPolicy).start();
             }
-            if (rolloverStrategy instanceof DirectFileRolloverStrategy) {
+            if (directWrite) {
                 // LOG4J2-2485: Initialize size from the most recently written file.
                 File file = new File(getFileName());
                 if (file.exists()) {
@@ -207,20 +219,47 @@ public class RollingFileManager extends FileManager {
             filePermissions, fileOwner, fileGroup, configuration), factory));
     }
 
+    /**
+     * Add a RolloverListener.
+     * @param listener The RolloverListener.
+     */
+    public void addRolloverListener(RolloverListener listener) {
+        rolloverListeners.add(listener);
+    }
+
+    /**
+     * Remove a RolloverListener.
+     * @param listener The RolloverListener.
+     */
+    public void removeRolloverListener(RolloverListener listener) {
+        rolloverListeners.remove(listener);
+    }
+
     /**
      * Returns the name of the File being managed.
      * @return The name of the File being managed.
      */
     @Override
     public String getFileName() {
-        if (rolloverStrategy instanceof DirectFileRolloverStrategy) {
+        if (directWrite) {
             fileName = ((DirectFileRolloverStrategy) rolloverStrategy).getCurrentFileName(this);
         }
         return fileName;
     }
+    
+    @Override
+    protected void createParentDir(File file) {
+        if (directWrite) {
+            file.getParentFile().mkdirs();
+        }
+    }
+
+    public boolean isDirectWrite() {
+        return directWrite;
+    }
 
     public FileExtension getFileExtension() {
-        return fileExtension;
+        return patternProcessor.getFileExtension();
     }
 
     // override to make visible for unit tests
@@ -323,22 +362,57 @@ public class RollingFileManager extends FileManager {
     }
 
     public synchronized void rollover(Date prevFileTime, Date prevRollTime) {
+        LOGGER.debug("Rollover PrevFileTime: {}, PrevRollTime: {}", prevFileTime.getTime(),
+                prevRollTime.getTime());
 		getPatternProcessor().setPrevFileTime(prevFileTime.getTime());
 		getPatternProcessor().setCurrentFileTime(prevRollTime.getTime());
 		rollover();
 	}
 
     public synchronized void rollover() {
-        if (!hasOutputStream()) {
+        if (!hasOutputStream() && !isCreateOnDemand() && !isDirectWrite()) {
             return;
         }
-        if (rollover(rolloverStrategy)) {
-            try {
-                size = 0;
-                initialTime = System.currentTimeMillis();
-                createFileAfterRollover();
-            } catch (final IOException e) {
-                logError("Failed to create file after rollover", e);
+        String currentFileName = fileName;
+        if (rolloverListeners.size() > 0) {
+            for (RolloverListener listener : rolloverListeners) {
+                try {
+                    listener.rolloverTriggered(currentFileName);
+                } catch (Exception ex) {
+                    LOGGER.warn("Rollover Listener {} failed with {}: {}", listener.getClass().getSimpleName(),
+                            ex.getClass().getName(), ex.getMessage());
+                }
+            }
+        }
+
+        boolean interrupted = Thread.interrupted(); // clear interrupted state
+        try {
+            if (interrupted) {
+                LOGGER.warn("RollingFileManager cleared thread interrupted state, continue to rollover");
+            }
+
+            if (rollover(rolloverStrategy)) {
+                try {
+                    size = 0;
+                    initialTime = System.currentTimeMillis();
+                    createFileAfterRollover();
+                } catch (final IOException e) {
+                    logError("Failed to create file after rollover", e);
+                }
+            }
+        } finally {
+            if (interrupted) { // restore interrupted state
+                Thread.currentThread().interrupt();
+            }
+        }
+        if (rolloverListeners.size() > 0) {
+            for (RolloverListener listener : rolloverListeners) {
+                try {
+                    listener.rolloverComplete(currentFileName);
+                } catch (Exception ex) {
+                    LOGGER.warn("Rollover Listener {} failed with {}: {}", listener.getClass().getSimpleName(),
+                            ex.getClass().getName(), ex.getMessage());
+                }
             }
         }
     }
@@ -371,10 +445,8 @@ public class RollingFileManager extends FileManager {
             if (policy instanceof LifeCycle) {
                 ((LifeCycle) policy).stop();
             }
-        } else {
-            if (triggeringPolicy instanceof LifeCycle) {
-                ((LifeCycle) triggeringPolicy).stop();
-            }
+        } else if (triggeringPolicy instanceof LifeCycle) {
+            ((LifeCycle) triggeringPolicy).stop();
         }
     }
 
@@ -397,6 +469,15 @@ public class RollingFileManager extends FileManager {
         return (T) this.triggeringPolicy;
     }
 
+    /**
+     * Package-private access for tests only.
+     *
+     * @return The semaphore that controls access to the rollover operation.
+     */
+    Semaphore getSemaphore() {
+        return semaphore;
+    }
+
     /**
      * Returns the rollover strategy.
      * @return The RolloverStrategy
@@ -617,12 +698,18 @@ public class RollingFileManager extends FileManager {
         }
     }
 
+    /**
+     * Updates the RollingFileManager's data during a reconfiguration. This method should be considered private.
+     * It is not thread safe and calling it outside of a reconfiguration may lead to errors. This method may be
+     * made protected in a future release.
+     * @param data The data to update.
+     */
     @Override
     public void updateData(final Object data) {
         final FactoryData factoryData = (FactoryData) data;
         setRolloverStrategy(factoryData.getRolloverStrategy());
-        setTriggeringPolicy(factoryData.getTriggeringPolicy());
         setPatternProcessor(new PatternProcessor(factoryData.getPattern(), getPatternProcessor()));
+        setTriggeringPolicy(factoryData.getTriggeringPolicy());
     }
 
     /**
@@ -639,12 +726,9 @@ public class RollingFileManager extends FileManager {
         @Override
         public RollingFileManager createManager(final String name, final FactoryData data) {
             long size = 0;
-            boolean writeHeader = !data.append;
             File file = null;
             if (data.fileName != null) {
                 file = new File(data.fileName);
-                // LOG4J2-1140: check writeHeader before creating the file
-                writeHeader = !data.append || !file.exists();
 
                 try {
                     FileUtils.makeParentDirs(file);
@@ -662,8 +746,9 @@ public class RollingFileManager extends FileManager {
                 final ByteBuffer buffer = ByteBuffer.wrap(new byte[actualSize]);
                 final OutputStream os = data.createOnDemand  || data.fileName == null ? null :
                         new FileOutputStream(data.fileName, data.append);
-                final long initialTime = data.createOnDemand || file == null ?
-                        0 : initialFileTime(file); // LOG4J2-531 create file first so time has valid value
+                // LOG4J2-531 create file first so time has valid value.
+                final long initialTime = file == null || !file.exists() ? 0 : initialFileTime(file);
+                final boolean writeHeader = file != null && file.exists() && file.length() == 0;
 
                 final RollingFileManager rm = new RollingFileManager(data.getLoggerContext(), data.fileName, data.pattern, os,
                     data.append, data.createOnDemand, size, initialTime, data.policy, data.strategy, data.advertiseURI,
