@@ -31,15 +31,20 @@ import org.apache.logging.log4j.core.Core;
 import org.apache.logging.log4j.core.Filter;
 import org.apache.logging.log4j.core.LogEvent;
 import org.apache.logging.log4j.core.LoggerContext;
+import org.apache.logging.log4j.core.appender.AbstractOutputStreamAppender;
+import org.apache.logging.log4j.core.appender.FileAppender;
 import org.apache.logging.log4j.core.async.AsyncLoggerConfig;
 import org.apache.logging.log4j.core.async.AsyncLoggerContext;
 import org.apache.logging.log4j.core.async.AsyncLoggerContextSelector;
 import org.apache.logging.log4j.core.config.plugins.Plugin;
 import org.apache.logging.log4j.core.config.plugins.PluginAttribute;
+import org.apache.logging.log4j.core.config.plugins.PluginBuilderAttribute;
+import org.apache.logging.log4j.core.config.plugins.PluginBuilderFactory;
 import org.apache.logging.log4j.core.config.plugins.PluginConfiguration;
 import org.apache.logging.log4j.core.config.plugins.PluginElement;
 import org.apache.logging.log4j.core.config.plugins.PluginFactory;
 import org.apache.logging.log4j.core.config.plugins.validation.constraints.Required;
+import org.apache.logging.log4j.core.config.properties.PropertiesConfiguration;
 import org.apache.logging.log4j.core.filter.AbstractFilterable;
 import org.apache.logging.log4j.core.impl.DefaultLogEventFactory;
 import org.apache.logging.log4j.core.impl.LocationAware;
@@ -99,6 +104,136 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
         }
     }
 
+    @PluginBuilderFactory
+    public static <B extends Builder<B>> B newBuilder() {
+        return new Builder<B>().asBuilder();
+    }
+
+    /**
+     * Builds LoggerConfig instances.
+     *
+     * @param <B>
+     *            The type to build
+     */
+    public static class Builder<B extends Builder<B>>
+            implements org.apache.logging.log4j.core.util.Builder<LoggerConfig> {
+
+        @PluginBuilderAttribute
+        private Boolean additivity;
+        @PluginBuilderAttribute
+        private Level level;
+        @PluginBuilderAttribute
+        private String levelAndRefs;
+        @PluginBuilderAttribute("name")
+        @Required(message = "Loggers cannot be configured without a name")
+        private String loggerName;
+        @PluginBuilderAttribute
+        private String includeLocation;
+        @PluginElement("AppenderRef")
+        private AppenderRef[] refs;
+        @PluginElement("Properties")
+        private Property[] properties;
+        @PluginConfiguration
+        private Configuration config;
+        @PluginElement("Filter")
+        private Filter filter;
+
+        public boolean isAdditivity() {
+            return additivity == null || additivity;
+        }
+
+        public B withAdditivity(boolean additivity) {
+            this.additivity = additivity;
+            return asBuilder();
+        }
+
+        public Level getLevel() {
+            return level;
+        }
+
+        public B withLevel(Level level) {
+            this.level = level;
+            return asBuilder();
+        }
+
+        public String getLevelAndRefs() {
+            return levelAndRefs;
+        }
+
+        public B withLevelAndRefs(String levelAndRefs) {
+            this.levelAndRefs = levelAndRefs;
+            return asBuilder();
+        }
+
+        public String getLoggerName() {
+            return loggerName;
+        }
+
+        public B withLoggerName(String loggerName) {
+            this.loggerName = loggerName;
+            return asBuilder();
+        }
+
+        public String getIncludeLocation() {
+            return includeLocation;
+        }
+
+        public B withIncludeLocation(String includeLocation) {
+            this.includeLocation = includeLocation;
+            return asBuilder();
+        }
+
+        public AppenderRef[] getRefs() {
+            return refs;
+        }
+
+        public B withRefs(AppenderRef[] refs) {
+            this.refs = refs;
+            return asBuilder();
+        }
+
+        public Property[] getProperties() {
+            return properties;
+        }
+
+        public B withProperties(Property[] properties) {
+            this.properties = properties;
+            return asBuilder();
+        }
+
+        public Configuration getConfig() {
+            return config;
+        }
+
+        public B withConfig(Configuration config) {
+            this.config = config;
+            return asBuilder();
+        }
+
+        public Filter getFilter() {
+            return filter;
+        }
+
+        public B withtFilter(Filter filter) {
+            this.filter = filter;
+            return asBuilder();
+        }
+
+        @Override
+        public LoggerConfig build() {
+            final String name = loggerName.equals(ROOT) ? Strings.EMPTY : loggerName;
+            LevelAndRefs container = LoggerConfig.getLevelAndRefs(level, refs, levelAndRefs, config);
+            boolean useLocation = includeLocation(includeLocation, config);
+            return new LoggerConfig(name, container.refs, filter, container.level, isAdditivity(), properties, config,
+                    useLocation);
+        }
+
+        @SuppressWarnings("unchecked")
+        public B asBuilder() {
+            return (B) this;
+        }
+    }
+
     /**
      * Default constructor.
      */
@@ -274,6 +409,14 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
         return level == null ? parent == null ? Level.ERROR : parent.getLevel() : level;
     }
 
+    /**
+     * Allows callers to determine the Level assigned to this LoggerConfig.
+     * @return the Level associated with this LoggerConfig or null if none is set.
+     */
+    public Level getExplicitLevel() {
+        return level;
+    }
+
     /**
      * Returns the LogEventFactory.
      *
@@ -381,33 +524,9 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
     @PerformanceSensitive("allocation")
     public void log(final String loggerName, final String fqcn, final Marker marker, final Level level,
             final Message data, final Throwable t) {
-        List<Property> props = null;
-        if (!propertiesRequireLookup) {
-            props = properties;
-        } else {
-            if (properties != null) {
-                props = new ArrayList<>(properties.size());
-                final LogEvent event = Log4jLogEvent.newBuilder()
-                        .setMessage(data)
-                        .setMarker(marker)
-                        .setLevel(level)
-                        .setLoggerName(loggerName)
-                        .setLoggerFqcn(fqcn)
-                        .setThrown(t)
-                        .build();
-                for (int i = 0; i < properties.size(); i++) {
-                    final Property prop = properties.get(i);
-                    final String value = prop.isValueNeedsLookup() // since LOG4J2-1575
-                            ? config.getStrSubstitutor().replace(event, prop.getValue()) //
-                            : prop.getValue();
-                    props.add(Property.createProperty(prop.getName(), value));
-                }
-            }
-        }
-        final LogEvent logEvent = logEventFactory instanceof LocationAwareLogEventFactory ?
-            ((LocationAwareLogEventFactory) logEventFactory).createEvent(loggerName, marker, fqcn, requiresLocation() ?
-                StackLocatorUtil.calcLocation(fqcn) : null, level, data, props, t) :
-            logEventFactory.createEvent(loggerName, marker, fqcn, level, data, props, t);
+        final List<Property> props = getProperties(loggerName, fqcn, marker, level, data, t);
+        final LogEvent logEvent = logEventFactory.createEvent(
+                loggerName, marker, fqcn, location(fqcn), level, data, props, t);
         try {
             log(logEvent, LoggerConfigPredicate.ALL);
         } finally {
@@ -416,6 +535,11 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
         }
     }
 
+    private StackTraceElement location(String fqcn) {
+        return requiresLocation() ?
+                StackLocatorUtil.calcLocation(fqcn) : null;
+    }
+
     /**
      * Logs an event.
      *
@@ -430,32 +554,8 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
     @PerformanceSensitive("allocation")
     public void log(final String loggerName, final String fqcn, final StackTraceElement location, final Marker marker,
         final Level level, final Message data, final Throwable t) {
-        List<Property> props = null;
-        if (!propertiesRequireLookup) {
-            props = properties;
-        } else {
-            if (properties != null) {
-                props = new ArrayList<>(properties.size());
-                final LogEvent event = Log4jLogEvent.newBuilder()
-                    .setMessage(data)
-                    .setMarker(marker)
-                    .setLevel(level)
-                    .setLoggerName(loggerName)
-                    .setLoggerFqcn(fqcn)
-                    .setThrown(t)
-                    .build();
-                for (int i = 0; i < properties.size(); i++) {
-                    final Property prop = properties.get(i);
-                    final String value = prop.isValueNeedsLookup() // since LOG4J2-1575
-                        ? config.getStrSubstitutor().replace(event, prop.getValue()) //
-                        : prop.getValue();
-                    props.add(Property.createProperty(prop.getName(), value));
-                }
-            }
-        }
-        final LogEvent logEvent = logEventFactory instanceof LocationAwareLogEventFactory ?
-            ((LocationAwareLogEventFactory) logEventFactory).createEvent(loggerName, marker, fqcn, location, level,
-                data, props, t) : logEventFactory.createEvent(loggerName, marker, fqcn, level, data, props, t);
+        final List<Property> props = getProperties(loggerName, fqcn, marker, level, data, t);
+        final LogEvent logEvent = logEventFactory.createEvent(loggerName, marker, fqcn, location, level, data, props, t);
         try {
             log(logEvent, LoggerConfigPredicate.ALL);
         } finally {
@@ -464,6 +564,45 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
         }
     }
 
+    private List<Property> getProperties(
+            final String loggerName,
+            final String fqcn,
+            final Marker marker,
+            final Level level,
+            final Message data,
+            final Throwable t) {
+        List<Property> snapshot = properties;
+        if (snapshot == null || !propertiesRequireLookup) {
+            return snapshot;
+        }
+        return getPropertiesWithLookups(loggerName, fqcn, marker, level, data, t, snapshot);
+    }
+
+    private List<Property> getPropertiesWithLookups(
+            final String loggerName,
+            final String fqcn,
+            final Marker marker,
+            final Level level,
+            final Message data,
+            final Throwable t,
+            final List<Property> props) {
+        List<Property> results = new ArrayList<>(props.size());
+        final LogEvent event = Log4jLogEvent.newBuilder()
+                .setMessage(data)
+                .setMarker(marker)
+                .setLevel(level)
+                .setLoggerName(loggerName)
+                .setLoggerFqcn(fqcn)
+                .setThrown(t)
+                .build();
+        for (int i = 0; i < props.size(); i++) {
+            final Property prop = props.get(i);
+            final String value = prop.evaluate(config.getStrSubstitutor()); // since LOG4J2-1575
+            results.add(Property.createProperty(prop.getName(), prop.getRawValue(), value));
+        }
+        return results;
+    }
+
     /**
      * Logs an event.
      *
@@ -504,6 +643,7 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
         logParent(event, predicate);
     }
 
+    @Override
     public boolean requiresLocation() {
         if (!includeLocation) {
             return false;
@@ -601,28 +741,26 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
      * @return A new LoggerConfig.
      * @since 2.6
      */
-    @PluginFactory
+    @Deprecated
     public static LoggerConfig createLogger(
-         // @formatter:off
-        @PluginAttribute(value = "additivity", defaultBoolean = true) final boolean additivity,
-        @PluginAttribute("level") final Level level,
-        @Required(message = "Loggers cannot be configured without a name") @PluginAttribute("name") final String loggerName,
-        @PluginAttribute("includeLocation") final String includeLocation,
-        @PluginElement("AppenderRef") final AppenderRef[] refs,
-        @PluginElement("Properties") final Property[] properties,
-        @PluginConfiguration final Configuration config,
-        @PluginElement("Filter") final Filter filter
-        // @formatter:on
+            // @formatter:off
+            @PluginAttribute(value = "additivity", defaultBoolean = true) final boolean additivity,
+            @PluginAttribute("level") final Level level,
+            @Required(message = "Loggers cannot be configured without a name") @PluginAttribute("name") final String loggerName,
+            @PluginAttribute("includeLocation") final String includeLocation,
+            @PluginElement("AppenderRef") final AppenderRef[] refs,
+            @PluginElement("Properties") final Property[] properties,
+            @PluginConfiguration final Configuration config,
+            @PluginElement("Filter") final Filter filter
+            // @formatter:on
     ) {
         final String name = loggerName.equals(ROOT) ? Strings.EMPTY : loggerName;
         return new LoggerConfig(name, Arrays.asList(refs), filter, level, additivity, properties, config,
-            includeLocation(includeLocation, config));
+                includeLocation(includeLocation, config));
     }
 
     /**
-     * @deprecated Please use {@link #includeLocation(String, Configuration)}
      */
-    @Deprecated
     protected static boolean includeLocation(final String includeLocationConfigValue) {
         return includeLocation(includeLocationConfigValue, null);
     }
@@ -654,7 +792,124 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
     @Plugin(name = ROOT, category = Core.CATEGORY_NAME, printObject = true)
     public static class RootLogger extends LoggerConfig {
 
-        @PluginFactory
+        @PluginBuilderFactory
+        public static <B extends Builder<B>> B newRootBuilder() {
+            return new Builder<B>().asBuilder();
+        }
+
+        /**
+         * Builds LoggerConfig instances.
+         *
+         * @param <B>
+         *            The type to build
+         */
+        public static class Builder<B extends Builder<B>>
+                implements org.apache.logging.log4j.core.util.Builder<LoggerConfig> {
+
+            @PluginBuilderAttribute
+            private boolean additivity;
+            @PluginBuilderAttribute
+            private Level level;
+            @PluginBuilderAttribute
+            private String levelAndRefs;
+            @PluginBuilderAttribute
+            private String includeLocation;
+            @PluginElement("AppenderRef")
+            private AppenderRef[] refs;
+            @PluginElement("Properties")
+            private Property[] properties;
+            @PluginConfiguration
+            private Configuration config;
+            @PluginElement("Filter")
+            private Filter filter;
+
+            public boolean isAdditivity() {
+                return additivity;
+            }
+
+            public B withAdditivity(boolean additivity) {
+                this.additivity = additivity;
+                return asBuilder();
+            }
+
+            public Level getLevel() {
+                return level;
+            }
+
+            public B withLevel(Level level) {
+                this.level = level;
+                return asBuilder();
+            }
+
+            public String getLevelAndRefs() {
+                return levelAndRefs;
+            }
+
+            public B withLevelAndRefs(String levelAndRefs) {
+                this.levelAndRefs = levelAndRefs;
+                return asBuilder();
+            }
+
+            public String getIncludeLocation() {
+                return includeLocation;
+            }
+
+            public B withIncludeLocation(String includeLocation) {
+                this.includeLocation = includeLocation;
+                return asBuilder();
+            }
+
+            public AppenderRef[] getRefs() {
+                return refs;
+            }
+
+            public B withRefs(AppenderRef[] refs) {
+                this.refs = refs;
+                return asBuilder();
+            }
+
+            public Property[] getProperties() {
+                return properties;
+            }
+
+            public B withProperties(Property[] properties) {
+                this.properties = properties;
+                return asBuilder();
+            }
+
+            public Configuration getConfig() {
+                return config;
+            }
+
+            public B withConfig(Configuration config) {
+                this.config = config;
+                return asBuilder();
+            }
+
+            public Filter getFilter() {
+                return filter;
+            }
+
+            public B withtFilter(Filter filter) {
+                this.filter = filter;
+                return asBuilder();
+            }
+
+            @Override
+            public LoggerConfig build() {
+                LevelAndRefs container = LoggerConfig.getLevelAndRefs(level, refs, levelAndRefs, config);
+                return new LoggerConfig(LogManager.ROOT_LOGGER_NAME, container.refs, filter, container.level,
+                        additivity, properties, config, includeLocation(includeLocation, config));
+            }
+
+            @SuppressWarnings("unchecked")
+            public B asBuilder() {
+                return (B) this;
+            }
+        }
+
+
+        @Deprecated
         public static LoggerConfig createLogger(
                 // @formatter:off
                 @PluginAttribute("additivity") final String additivity,
@@ -668,12 +923,47 @@ public class LoggerConfig extends AbstractFilterable implements LocationAware {
             final List<AppenderRef> appenderRefs = Arrays.asList(refs);
             final Level actualLevel = level == null ? Level.ERROR : level;
             final boolean additive = Booleans.parseBoolean(additivity, true);
-
             return new LoggerConfig(LogManager.ROOT_LOGGER_NAME, appenderRefs, filter, actualLevel, additive,
                     properties, config, includeLocation(includeLocation, config));
         }
     }
 
+    protected static LevelAndRefs getLevelAndRefs(Level level, AppenderRef[] refs, String levelAndRefs,
+            Configuration config) {
+        LevelAndRefs result = new LevelAndRefs();
+        if (levelAndRefs != null) {
+            if (config instanceof PropertiesConfiguration) {
+                if (level != null) {
+                    LOGGER.warn("Level is ignored when levelAndRefs syntax is used.");
+                }
+                if (refs != null && refs.length > 0) {
+                    LOGGER.warn("Appender references are ignored when levelAndRefs syntax is used");
+                }
+                final String[] parts = Strings.splitList(levelAndRefs);
+                result.level = Level.getLevel(parts[0]);
+                if (parts.length > 1) {
+                    List<AppenderRef> refList = new ArrayList<>();
+                    Arrays.stream(parts).skip(1).forEach((ref) ->
+                            refList.add(AppenderRef.createAppenderRef(ref, null, null)));
+                    result.refs = refList;
+                }
+            } else {
+                LOGGER.warn("levelAndRefs are only allowed in a properties configuration. The value is ignored.");
+                result.level = level;
+                result.refs = refs != null ? Arrays.asList(refs) : new ArrayList<>();
+            }
+        } else {
+            result.level = level;
+            result.refs = refs != null ? Arrays.asList(refs) : new ArrayList<>();
+        }
+        return result;
+    }
+
+    protected static class LevelAndRefs {
+        public Level level;
+        public List<AppenderRef> refs;
+    }
+
     protected enum LoggerConfigPredicate {
         ALL() {
             @Override
