@@ -16,13 +16,12 @@
  */
 package org.apache.logging.slf4j;
 
+import static org.junit.Assert.fail;
+
 import org.apache.logging.log4j.LoggingException;
 import org.junit.Test;
-import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.junit.Assert.fail;
-
 /**
  * Tests StackOverflow when slf4j-impl and to-slf4j are both present.
  */
@@ -31,7 +30,7 @@ public class OverflowTest {
     @Test
     public void log() {
         try {
-            final Logger logger = LoggerFactory.getLogger(OverflowTest.class);
+            LoggerFactory.getLogger(OverflowTest.class);
             fail("Failed to detect inclusion of log4j-to-slf4j");
         } catch (LoggingException ex) {
             // Expected exception.
