@@ -39,8 +39,9 @@ public class SpringLookupTest {
         env.setDefaultProfiles("one", "two");
         env.setProperty("app.property", "test");
         LoggerContext context = (LoggerContext) LogManager.getContext(false);
-        context.putObject(Log4j2CloudConfigLoggingSystem.ENVIRONMENT_KEY, env);
+        context.putObject(Log4j2SpringBootLoggingSystem.ENVIRONMENT_KEY, env);
         SpringLookup lookup = new SpringLookup();
+        lookup.setLoggerContext(context);
         String result = lookup.lookup("profiles.active");
         assertNotNull("No active profiles", result);
         assertEquals("Incorrect active profile", "test", result);
@@ -66,9 +67,10 @@ public class SpringLookupTest {
         env.setActiveProfiles("test");
         env.setProperty("app.property", "test");
         LoggerContext context = (LoggerContext) LogManager.getContext(false);
-        context.putObject(Log4j2CloudConfigLoggingSystem.ENVIRONMENT_KEY, env);
-
-        StrLookup lookup = new Interpolator();
+        context.putObject(Log4j2SpringBootLoggingSystem.ENVIRONMENT_KEY, env);
+        Interpolator lookup = new Interpolator();
+        lookup.setConfiguration(context.getConfiguration());
+        lookup.setLoggerContext(context);
         String result = lookup.lookup("spring:profiles.active");
         assertNotNull("No active profiles", result);
         assertEquals("Incorrect active profile", "test", result);
